"use strict";

const {
  URL
} = require(`url`);

const {
  createRemoteFileNode
} = require(`gatsby-source-filesystem`);

const path = require(`path`);

const probeImageSize = require(`probe-image-size`);

const {
  getOptions
} = require(`./plugin-options`);

const getHref = link => {
  if (typeof link === `object`) {
    return link.href;
  }

  return link;
};

exports.getHref = getHref;
/**
 * This FN takes in node data and returns Gatsby Image CDN fields that should be added to that node. If the input node isn't an image an empty object is returned.
 */

const getGatsbyImageCdnFields = async ({
  node,
  type,
  pluginOptions,
  fileNodesExtendedData,
  reporter
}) => {
  const isFile = isFileNode({
    internal: {
      type
    }
  });

  if (!isFile) {
    return {};
  }

  const mimeType = node.attributes.filemime;

  if (!mimeType || !mimeType.includes(`image/`)) {
    return {};
  }

  const url = pluginOptions.baseUrl + getFileUrl(node.attributes);
  const extraNodeData = (fileNodesExtendedData === null || fileNodesExtendedData === void 0 ? void 0 : fileNodesExtendedData.get(node.id)) || null;

  try {
    var _node$attributes;

    const imageSize = extraNodeData || (await probeImageSize(url));
    const requiredData = imageSize && imageSize.width && imageSize.height && url;

    if (!requiredData) {
      return {};
    }

    const gatsbyImageCdnFields = {
      filename: (_node$attributes = node.attributes) === null || _node$attributes === void 0 ? void 0 : _node$attributes.filename,
      url,
      placeholderUrl: url,
      width: imageSize.width,
      height: imageSize.height,
      mimeType
    };
    return gatsbyImageCdnFields;
  } catch (e) {
    reporter.error(e);
    reporter.info(JSON.stringify({
      extraNodeData,
      url,
      attributes: node.attributes
    }, null, 2));
    reporter.panic(`Encountered an unrecoverable error while generating Gatsby Image CDN fields. See above for additional information.`);
  }

  return {};
};

const nodeFromData = async (datum, createNodeId, entityReferenceRevisions = [], pluginOptions, fileNodesExtendedData, reporter) => {
  const {
    attributes: {
      id: attributeId,
      ...attributes
    } = {}
  } = datum;
  const preservedId = typeof attributeId !== `undefined` ? {
    _attributes_id: attributeId
  } : {};
  const langcode = attributes.langcode || `und`;
  const type = datum.type.replace(/-|__|:|\.|\s/g, `_`);
  const gatsbyImageCdnFields = getGatsbyImageCdnFields({
    node: datum,
    type,
    pluginOptions,
    fileNodesExtendedData,
    reporter
  });
  return {
    id: createNodeId(createNodeIdWithVersion(datum.id, datum.type, langcode, attributes.drupal_internal__revision_id, entityReferenceRevisions)),
    drupal_id: datum.id,
    parent: null,
    drupal_parent_menu_item: attributes.parent,
    children: [],
    ...attributes,
    ...preservedId,
    ...gatsbyImageCdnFields,
    drupal_relationships: datum.relationships,
    relationships: {},
    internal: {
      type
    }
  };
};

exports.nodeFromData = nodeFromData;

const isEntityReferenceRevision = (type, entityReferenceRevisions = []) => entityReferenceRevisions.findIndex(revisionType => type.indexOf(revisionType) === 0) !== -1;

const createNodeIdWithVersion = (id, type, langcode, revisionId, entityReferenceRevisions = []) => {
  var _getOptions, _getOptions$languageC, _getOptions$languageC2;

  // Fallback to default language for entities that don't translate.
  if ((_getOptions = getOptions()) !== null && _getOptions !== void 0 && (_getOptions$languageC = _getOptions.languageConfig) !== null && _getOptions$languageC !== void 0 && _getOptions$languageC.nonTranslatableEntities.includes(type)) {
    langcode = getOptions().languageConfig.defaultLanguage;
  } // If the source plugin hasn't enabled `translation` then always just set langcode
  // to "undefined".


  let langcodeNormalized = getOptions().languageConfig ? langcode : `und`;

  if (getOptions().languageConfig && !((_getOptions$languageC2 = getOptions().languageConfig) !== null && _getOptions$languageC2 !== void 0 && _getOptions$languageC2.enabledLanguages.includes(langcodeNormalized))) {
    langcodeNormalized = getOptions().languageConfig.defaultLanguage;
  } // The relationship between an entity and another entity also depends on the revision ID if the field is of type
  // entity reference revision such as for paragraphs.


  return isEntityReferenceRevision(type, entityReferenceRevisions) ? `${langcodeNormalized}.${id}.${revisionId || 0}` : `${langcodeNormalized}.${id}`;
};

exports.createNodeIdWithVersion = createNodeIdWithVersion;

const isFileNode = node => {
  var _node$internal;

  const type = node === null || node === void 0 ? void 0 : (_node$internal = node.internal) === null || _node$internal === void 0 ? void 0 : _node$internal.type;
  return type === `files` || type === `file__file`;
};

exports.isFileNode = isFileNode;

const getFileUrl = node => {
  let fileUrl = node.url;

  if (typeof node.uri === `object`) {
    // Support JSON API 2.x file URI format https://www.drupal.org/node/2982209
    fileUrl = node.uri.url;
  }

  return fileUrl;
};

exports.downloadFile = async ({
  node,
  store,
  cache,
  createNode,
  createNodeId,
  getCache,
  reporter
}, {
  basicAuth,
  baseUrl
}) => {
  // handle file downloads
  if (isFileNode(node)) {
    let fileType;

    if (typeof node.uri === `object`) {
      // get file type from uri prefix ("S3:", "public:", etc.)
      const uriPrefix = node.uri.value.match(/^\w*:/);
      fileType = uriPrefix ? uriPrefix[0] : null;
    }

    const fileUrl = getFileUrl(node, baseUrl); // Resolve w/ baseUrl if node.uri isn't absolute.

    const url = new URL(fileUrl, baseUrl); // If we have basicAuth credentials, add them to the request.

    const basicAuthFileSystems = [`public:`, `private:`, `temporary:`];
    const auth = typeof basicAuth === `object` && basicAuthFileSystems.includes(fileType) ? {
      htaccess_user: basicAuth.username,
      htaccess_pass: basicAuth.password
    } : {};
    const fileNode = await createRemoteFileNode({
      url: url.href,
      name: path.parse(decodeURIComponent(url.pathname)).name,
      store,
      cache,
      createNode,
      createNodeId,
      getCache,
      parentNodeId: node.id,
      auth,
      reporter
    });

    if (fileNode) {
      node.localFile___NODE = fileNode.id;
    }
  }
};